module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::EdgeMethods
    include RepositoriesProxy

    def increment_views(user, learning_object)
      if !edge_exists? "Views", user.rid, learning_object.id
        create_edge "Views", user.rid, learning_object.id
        learning_object.views = learning_object.views + 1
      end
    end

    def liked?(user, learning_object)
      edge_exists? 'Likes', user.rid, learning_object.id
    end

    #
    # Usage:
    #   repository.for(:learning_objects).like current_user, @learning_object
    #
    def like(user, learning_object)
      like_edge_class = 'Likes'

      if liked? user, learning_object
        raise 'The user already likes this object.'
      end

      create_edge like_edge_class, user.rid, learning_object.id
      learning_object.likes = learning_object.likes + 1
    end

    def dislike(user, learning_object)
      like_edge_class = 'Likes'

      if liked? user, learning_object
        destroy_edge like_edge_class, user.rid, learning_object.id
        learning_object.likes = learning_object.likes - 1

        return true
      end

      raise 'The user hasn`t likes this object.'
    end

    # Example:
    #   list = repository.for(:learning_objects).all
    #   list.each do |learning_object|
    #     learning_object.inspect <LearningObject model>
    #   end
    def all
      learning_objects_hash = connection.query "SELECT FROM LearningObject"
      build_objects(learning_objects_hash) || []
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def get_by_dspace_id(id_dspace)
      result = select_by_property(odb_class, "id_dspace", id_dspace)
      build_object result.first
    end

    def get_subjects(learning_object)
      result = get_edges_end("IsAbout", "out", learning_object.id)
      subject_repository.build_objects(result)
    end

    def get_attributes(learning_object)
      result = get_edges_end("HasAttr", "out", learning_object.id)
      attribute_repository.build_objects(result)
    end

    def create_relations(learning_object)
      edges = []
      unless learning_object.subjects.empty?
        edges << create_edges_from_array("IsAbout", learning_object.id, learning_object.subjects, true)
      end
      unless learning_object.attributes.empty?
        edges << create_edges_from_array("HasAttr", learning_object.id, learning_object.attributes, true)
      end
      edges.flatten
    end

    def save(learning_object)
      result = connection.command "INSERT INTO LearningObject CONTENT #{learning_object.to_json}"
    end

    def update_property(learning_object, property, value)
      if accepted_properties.include? property
        connection.command "UPDATE LearningObject SET #{property}='#{value}' WHERE @rid = #{learning_object.id}"
      end
    end

    ##
    # To create "index:learningobject_search" on OrientDB, use the following command:
    #   CREATE INDEX learningobject_search
    #   ON LearningObject (name, description)
    #   FULLTEXT ENGINE LUCENE
    #   METADATA {"analyzer":"org.apache.lucene.analysis.br.BrazilianAnalyzer"}
    def search(qry)
      learning_objects_hash = connection.query "
                                SELECT EXPAND(rid)
                                FROM index:learningobject_search
                                WHERE key LUCENE '#{qry}'
                              ", limit: 10000
      build_objects(learning_objects_hash) || []
    end

    def build_object(args={})
      lo = nil
      unless args.nil?
        lo = LearningObject.new(:id => args["@rid"],
                                :name => args["name"],
                                :description => args["description"],
                                :thumbnail => args["thumbnail"],
                                :created_at => args["created_at"],
                                :id_dspace => args["id_dspace"],
                                :type => args["type"],
                                :bitstreams => args["bitstreams"],
                                :metadata => args["metadata"],
                                :last_modified => args["last_modified"])
        lo.likes = args.has_key?("in_Likes") ? args["in_Likes"].size : 0
        lo.views = args.has_key?("in_Views") ? args["in_Views"].size : 0
      end
      lo
    end

    def build_hash
      hash = super
      hash.delete("likes")
      hash.delete("views")
      hash.delete("downloads")
      hash.delete("subjects")
      hash.delete("attributes")
      hash
    end

    private

    def accepted_properties
      ['thumbnail']
    end

    def odb_class
      "LearningObject"
    end

    def count_likes(learning_object)
      get_in_edges_count "Likes", learning_object.id
    end

    def count_views(learning_object)
      get_in_edges_count "Views", learning_object.id
    end

  end
end
