require 'dspace_rest_client'

namespace :dspace do
  desc "Dspace tasks"

  task import: :environment do
    desc "Import DSpace items to LearningObject Repositoy"

    include RepositoriesProxy

    # Quantity of items fetched on each iteration
    limit = 500

    # Start point from where items will be fetched
    offset = 0

    loop do

      puts " --> importing items from #{offset} to #{offset+limit}"

      begin
        # Get items from dspace (from offset to offset+limit)
        dspace_items = get_items_from_dspace(limit,offset)
      rescue
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if dspace_items.empty?

        # Increment offset, to get new items on next iteration
        offset = offset + limit

        # Iterates through items
        # Verifies if item is already on repository
        # Initializes new LearningObjects
        # and saves them on LearningObjects's repository
        dspace_items.each do |item|
          result = learning_object_repository.get_by_dspace_id item.id
          if result.nil?
            learning_object = initialize_learning_object item
            learning_object_repository.save learning_object
          end
        end

      end

    end

  end

  task sync: :environment do
    desc "Synchronize DSpace items to LearningObject Repositoy"

    include RepositoriesProxy

    # Quantity of items fetched on each iteration
    limit = 500

    # Start point from where items will be fetched
    offset = 0

    loop do

      puts " --> Verifying items from #{offset} to #{offset+limit}"

      begin
        # Get items from dspace (from offset to offset+limit)
        items = learning_object_repository.all_from_offset_to_limit(offset,limit)
      rescue
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if items.empty?

        # Increment offset, to get new items on next iteration
        offset = offset + limit

        item_ids = []

        items.each do |item|
          item_ids << {id: item.id, id_dspace: item.id_dspace}
        end

        MassiveDspaceSyncWorker.perform_async(item_ids)

      end
    end

  end

  private

  def dspace_repository
    @dspace_repository ||= Dspace::Client.instance.repository.item_repository
  end

  def get_items_from_dspace(limit, offset)
    dspace_repository.get_all_items(
      expand: ['metadata','bitstreams'],
      limit: limit, offset: offset
    )
  end

  def initialize_learning_object item
    metadata = build_array_of(item.metadata)
    bitstreams = build_array_of(item.bit_streams)
    current_date = Time.new

    LearningObject.new(
      :name => item.name,
      :description => select_value_of(metadata, "dc.description"),
      :thumbnail => '/thumbnails/default_thumbnail.jpg',
      :created_at => current_date,
      :last_modified => current_date,
      :id_dspace => item.id,
      :type => item.type,
      :bitstreams => bitstreams,
      :metadata => metadata
    )
  end

  def build_array_of(item_content=[])
    return item_content if item_content.nil?
    content_array = []
    item_content.each do |i|
      content_array << i.to_h
    end
    return content_array
  end

  def select_value_of(array, key)
    descriptions = array.select { |a|  a[:key] == key }
    unless descriptions.empty?
      descriptions.first[:value]
    end
  end

end
